@@ -1,12 +1,29 @@
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals
 
 from django.db.backends import BaseDatabaseIntrospection
 
+try:
+    # Added with Django 1.7
+    from django.db.backends import FileInfo
+except ImportError:
+    from collections import namedtuple
+    # Structure returned by the DB-API cursor.description interface (PEP 249)
+    FieldInfo = namedtuple('FieldInfo',
+        'name type_code display_size internal_size precision scale null_ok')
+
 AUTO_FIELD_MARKER = -1000
 BIG_AUTO_FIELD_MARKER = -1001
 MONEY_FIELD_MARKER = -1002
 
 class BaseSqlDatabaseIntrospection(BaseDatabaseIntrospection):
+    def get_field_type(self, data_type, description):
+        field_type = self.data_types_reverse[data_type]
+        if (field_type == 'CharField'
+                and description.internal_size is not None
+                and description.internal_size > 8000):
+            field_type = 'TextField'
+        return field_type
+
     def get_table_list(self, cursor):
         "Return a list of table and view names in the current database."
         cursor.execute("SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_TYPE = 'BASE TABLE' UNION SELECT TABLE_NAME FROM INFORMATION_SCHEMA.VIEWS")
@@ -28,8 +45,12 @@ class BaseSqlDatabaseIntrospection(BaseDatabaseIntrospection):
         Return a dict mapping field name to data type. DB-API cursor description 
         interprets the date columns as chars.
         """
-        cursor.execute('SELECT [COLUMN_NAME], [DATA_TYPE] FROM INFORMATION_SCHEMA.COLUMNS WHERE [TABLE_NAME] LIKE \'%s\'' % table_name)
-        results = dict(cursor.fetchall())
+        cursor.execute('''
+SELECT [COLUMN_NAME], [DATA_TYPE], [CHARACTER_MAXIMUM_LENGTH]
+FROM INFORMATION_SCHEMA.COLUMNS
+WHERE [TABLE_NAME] LIKE \'%s\'
+        ''' % table_name)
+        results = dict([(c[0], (c[1], c[2])) for c in cursor.fetchall()])
         return results
 
     def get_table_description(self, cursor, table_name, identity_check=True):
@@ -48,7 +69,7 @@ class BaseSqlDatabaseIntrospection(BaseDatabaseIntrospection):
         items = list()
         for column in columns:
             column = list(column) # Convert tuple to list
-            items.append(column)
+            items.append(FieldInfo(*column))
         return items
 
     def _name_to_index(self, cursor, table_name):
@@ -73,6 +94,12 @@ class BaseSqlDatabaseIntrospection(BaseDatabaseIntrospection):
 
     # django 1.6 version
     def get_key_columns(self, cursor, table_name):
+        """
+        Backends can override this to return a list of (column_name, referenced_table_name,
+        referenced_column_name) for all key columns in given table.
+        """
+        source_field_dict = self._name_to_index(cursor, table_name)
+
         sql = """
 select
     COLUMN_NAME = fk_cols.COLUMN_NAME,
@@ -100,7 +127,13 @@ where
 	fk.TABLE_NAME = %s"""
 
         cursor.execute(sql,[table_name])
-        return list(cursor.fetchall())
+        relations = cursor.fetchall()
+
+        key_columns = []
+        key_columns.extend([(source_column, target_table, target_column) \
+            for source_column, target_table, target_column in relations])
+        return key_columns
+
 
     def get_indexes(self, cursor, table_name):
     #    Returns a dictionary of fieldname -> infodict for the given table,
@@ -205,20 +238,30 @@ where
                           }
             constraints[name] = constraint
 
-        # getting check constraints
+        # get check constraints
         sql = """
-        select object_id, name
-        from sys.check_constraints where parent_object_id = OBJECT_ID(%s)
+        SELECT kc.constraint_name, kc.column_name
+        FROM information_schema.constraint_column_usage AS kc
+        JOIN information_schema.table_constraints AS c ON
+            kc.table_schema = c.table_schema AND
+            kc.table_name = c.table_name AND
+            kc.constraint_name = c.constraint_name
+        WHERE
+            c.constraint_type = 'CHECK'
+            AND
+            kc.table_name = %s
         """
         cursor.execute(sql,[table_name])
-        for id, name, type in list(cursor.fetchall()):
-            constraint = {"columns": [],
-                          "primary_key": False,
-                          "unique": False,
-                          "index": False,
-                          "check": True,
-                          "foreign_key": None,
-                          }
-            constraints[name] = constraint
+        for constraint, column in list(cursor.fetchall()):
+            if column not in constraints:
+                constraints[constraint] = {
+                    "columns": [],
+                    "primary_key": False,
+                    "unique": False,
+                    "index": False,
+                    "check": True,
+                    "foreign_key": None,
+                    }
+            constraints[constraint]['columns'].append(column)
 
         return constraints
