@@ -123,7 +123,7 @@ class DatabaseWrapper(SqlServerBaseWrapper):
             # Use a cursor directly, bypassing Django's utilities.
             with self.connection.cursor() as cursor:
                 cursor.execute("SELECT 1")
-        except self.Database.Error:
+        except:
             return False
         else:
             return True
